import { ConflictException, Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { PassportStrategy } from '@nestjs/passport';
import { Strategy, ExtractJwt } from 'passport-jwt';
import { ModuloService } from 'src/modulo/modulo.service';
import { OperadorService } from 'src/operador/operador.service';
import { UsuarioService } from 'src/usuario/usuario.service';
import { Modulo } from '../../modulo/entity/modulo.entity';
import { Operador } from 'src/operador/entity/operador.entity';
import { Usuario } from 'src/usuario/entity/usuario.entity';
import { JwtPayload } from '../dto/jwt-payload';

@Injectable()
export class JwtStrategyService extends PassportStrategy(Strategy) {
  constructor(
    private configService: ConfigService,
    private moduloService: ModuloService,
    private operadorService: OperadorService,
    private usuarioService: UsuarioService,
  ) {
    super({
      jwtFromRequest: ExtractJwt.fromAuthHeaderAsBearerToken(),
      secretOrKey: configService.get<string>('AUTH_SECRETKEY'),
    });
  }

  async validate(payload: JwtPayload) {
    const user: { modulo?: Modulo; usuario?: Usuario; operador?: Operador } =
      {};

    if (payload.id_usuario)
      await this.usuarioService
        .findById(payload.id_usuario, true, true)
        .then((usuario) => (user.usuario = usuario));
    else if (payload.id_operador)
      await this.operadorService
        .findById(payload.id_operador)
        .then(async (operador) => {
          if (!operador.activo)
            throw new ConflictException('Esta cuenta esta desactivada.');
          if (operador.tipoUsuario.id_tipo_usuario === 4) {
            if (payload.id_modulo)
              await this.moduloService
                .findById(payload.id_modulo)
                .then((modulo) => (user.modulo = modulo));
            else
              throw new ConflictException(
                'Credenciales no válidas, inicia sesión de nuevo.',
              );
          }
          user.operador = operador;
        });
    return user;
  }
}
