import { ConflictException, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Equipo } from '../equipo/entity/equipo.entity';
import { EquipoTipoEntrada } from './entity/equipo-tipo-entrada.entity';
import { TipoEntrada } from '../institucion-tipo-entrada/entity/tipo-entrada.entity';
import { EquipoService } from '../equipo/equipo.service';
import { InstitucionTipoEntradaService } from '../institucion-tipo-entrada/institucion-tipo-entrada.service';

@Injectable()
export class EquipoTipoEntradaService {
  constructor(
    @InjectRepository(EquipoTipoEntrada)
    private repository: Repository<EquipoTipoEntrada>,
    private equipoService: EquipoService,
    private institucionTipoEntradaService: InstitucionTipoEntradaService,
  ) {}

  async create(
    id_equipo: number | Equipo,
    id_tipo_entrada: number | TipoEntrada,
  ) {
    const equipo =
      typeof id_equipo === 'number'
        ? await this.equipoService.findById(id_equipo)
        : id_equipo;
    const tipoEntrada =
      typeof id_tipo_entrada === 'number'
        ? await this.institucionTipoEntradaService.findTipoEntradaById(
            id_tipo_entrada,
          )
        : id_tipo_entrada;

    return this.findEquipoTipoEntradaByEquipoTipoEntrada(equipo, tipoEntrada)
      .then((_) =>
        this.repository.save(this.repository.create({ equipo, tipoEntrada })),
      )
      .then((equipoTipoEntrada) => ({
        message: `Se asignó el tipo de entrada: ${tipoEntrada.tipo_entrada}, al equipo con número de inventario: ${equipo.numero_inventario}.`,
        equipoTipoEntrada,
      }));
  }

  delete(id_equipo_tipo_entrada: number) {
    return this.findById(id_equipo_tipo_entrada)
      .then((equipoTipoEntrada) => this.repository.remove(equipoTipoEntrada))
      .then((_) => ({
        message: 'Se eliminó correctamente el tipo de entrada de este equipo.',
      }));
  }

  findById(id_equipo_tipo_entrada: number) {
    return this.repository
      .findOne({ id_equipo_tipo_entrada })
      .then((equipoTipoEntrada) => {
        if (!equipoTipoEntrada)
          throw new ConflictException('No existe este id equipo tipo entrada.');
        return equipoTipoEntrada;
      });
  }

  findEquipoTipoEntradaByEquipoTipoEntrada(
    equipo: Equipo,
    tipoEntrada: TipoEntrada,
    validarExistencia = true,
  ) {
    return this.repository
      .findOne({ equipo, tipoEntrada })
      .then((equipoTipoEntrada) => {
        if (validarExistencia && equipoTipoEntrada)
          throw new ConflictException(
            'Este tipo de entrada ya fue asignado a este equipo.',
          );
        return equipoTipoEntrada;
      });
  }
}
