import * as moment from 'moment';
import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Institucion } from '../institucion/entity/institucion.entity';
import { InstitucionDia } from './entity/institucion-dia.entity';
import { Operador } from 'src/operador/entity/operador.entity';
import { InstitucionService } from '../institucion/institucion.service';

@Injectable()
export class InstitucionDiaService {
  constructor(
    @InjectRepository(InstitucionDia)
    private repository: Repository<InstitucionDia>,
    private institucionService: InstitucionService,
  ) {}

  findAllByIdInstitucion(id_institucion: number) {
    return this.institucionService
      .findById(id_institucion)
      .then((institucion) =>
        this.repository.find({ where: { institucion }, order: { dia: 'ASC' } }),
      );
  }

  findById(id_institucion_dia: number) {
    return this.repository
      .findOne({
        join: {
          alias: 'id',
          innerJoinAndSelect: {
            d: 'id.dia',
            i: 'id.institucion',
          },
        },
        where: { id_institucion_dia },
      })
      .then((institucionDia) => {
        if (!institucionDia)
          throw new NotFoundException('No existe este id institución día.');
        return institucionDia;
      });
  }

  findByInstitucionDia(institucion: Institucion, id_dia: number) {
    return this.repository.findOne({ institucion, dia: { id_dia } });
  }

  async hoy(id_institucion: number) {
    const ahora = moment();

    if (ahora.weekday() === 0 || ahora.weekday() === 6)
      throw new ConflictException(
        'No se puede pedir equipo de cómputo los sabados y domingos.',
      );
    return this.institucionService
      .findById(id_institucion)
      .then((institucion) =>
        this.findByInstitucionDia(institucion, ahora.weekday()),
      );
  }

  update(admin: Operador, attrs: Partial<InstitucionDia>) {
    return this.findById(attrs.id_institucion_dia)
      .then((institucionDia) => {
        if (
          admin.institucion.id_institucion !=
          institucionDia.institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes actualizar la información de este día porque no le pertenece a tu institución.',
          );
        Object.assign(institucionDia, attrs);
        return this.repository.save(institucionDia);
      })
      .then((_) => ({
        message: 'Se guardaron los cambios correctamente.',
      }));
  }
}
