import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Infraccion } from './entity/infraccion.entity';
import { InstitucionInfraccion } from './entity/institucion-infraccion.entity';
import { Operador } from 'src/operador/entity/operador.entity';
import { InstitucionService } from '../institucion/institucion.service';

@Injectable()
export class InstitucionInfraccionService {
  constructor(
    @InjectRepository(Infraccion)
    private infraccionRepository: Repository<Infraccion>,
    @InjectRepository(InstitucionInfraccion)
    private institucionInfraccionRepository: Repository<InstitucionInfraccion>,
    private institucionService: InstitucionService,
  ) {}

  create(infraccion: string) {
    return this.infraccionRepository
      .findOne({ infraccion })
      .then((existeInfraccion) => {
        if (existeInfraccion)
          throw new ConflictException('Ya existe esta infracción.');
        return this.infraccionRepository.save(
          this.infraccionRepository.create({ infraccion }),
        );
      })
      .then(async (infraccion) => {
        const instituciones = await this.institucionService.findAll();

        for (let i = 0; i < instituciones.length; i++)
          await this.institucionInfraccionRepository.save(
            this.institucionInfraccionRepository.create({
              infraccion,
              institucion: instituciones[i],
            }),
          );
        return { message: 'Se creó correctamente una nueva infracción.' };
      });
  }

  findAll() {
    return this.infraccionRepository.find({ order: { infraccion: 'ASC' } });
  }

  findAllByIdInstitucion(id_institucion: number) {
    return this.institucionService
      .findById(id_institucion)
      .then((institucion) =>
        this.institucionInfraccionRepository
          .createQueryBuilder('ii')
          .innerJoinAndSelect('ii.infraccion', 'i')
          .innerJoinAndSelect('ii.institucion', 'in')
          .where('in.id_institucion = :id_institucion', {
            id_institucion: institucion.id_institucion,
          })
          .orderBy('i.infraccion')
          .getMany(),
      );
  }

  findById(id_institucion_infraccion: number) {
    return this.institucionInfraccionRepository
      .findOne({
        join: {
          alias: 'ii',
          innerJoinAndSelect: { i: 'ii.institucion', in: 'ii.infraccion' },
        },
        where: { id_institucion_infraccion },
      })
      .then((institucionInfraccion) => {
        if (!institucionInfraccion)
          throw new NotFoundException(
            'No existe este id institución infracción.',
          );
        return institucionInfraccion;
      });
  }

  update(admin: Operador, attrs: Partial<InstitucionInfraccion>) {
    return this.findById(attrs.id_institucion_infraccion)
      .then((institucionInfraccion) => {
        if (
          admin.institucion.id_institucion !=
          institucionInfraccion.institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes actualizar la información de esta infraccion porque no le pertenece a tu institución.',
          );
        Object.assign(institucionInfraccion, attrs);
        return this.institucionInfraccionRepository.save(institucionInfraccion);
      })
      .then((_) => ({
        message: 'Se guardaron los cambios correctamente.',
      }));
  }
}
