import { Controller, Get, Query, UseGuards } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { MotivoService } from './motivo.service';
import { IdEquipoPaginaDto } from '../dto/id-equipo-pagina.dto';
import { MotivoOutputDto } from './dto/output/motivo.dto';

@Controller('motivo')
@ApiTags('motivo')
export class MotivoController {
  constructor(private motivoService: MotivoService) {}

  @Serealize(MotivoOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que retorna 25 cambios de status que ha tenido el equipo dependiendo de la página en la que se encuentra el alumno.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id del equipo.',
    name: 'id_equipo',
    type: 'string',
  })
  @ApiQuery({
    description: 'Página en la que se encuentra el usuario.',
    name: 'pagina',
    type: 'string',
  })
  get(@Query() query: IdEquipoPaginaDto) {
    return this.motivoService.findAllByIdEquipo(
      parseInt(query.id_equipo),
      parseInt(query.pagina),
    );
  }

  // @Get('reporte')
  // reporte() {}
}
