import * as sha1 from 'sha1';
import * as md5 from 'md5';
import { Injectable, Inject } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { Client } from 'nestjs-soap';

@Injectable()
export class SoapClientService {
  constructor(
    @Inject('DGAE') private readonly soapDgae: Client,
    @Inject('DGP') private readonly soapDpg: Client,
    private configService: ConfigService,
  ) {}

  dgae(cta: string, plt: string, crr: string, nvl: string) {
    return new Promise((resolve, reject) => {
      this.soapDgae.return_vigencia(
        {
          key: sha1(this.configService.get<string>('SOAP_DGAE_KEY')),
          cta,
          plt,
          crr,
          nvl,
        },
        (err, res) => (err ? reject(err) : resolve(res)),
      );
    });
  }

  dgp(rfcNumEmp: string) {
    return new Promise((resolve, reject) => {
      this.soapDpg.PlazaTrab(
        {
          rfcNumEmp,
          usuario: this.configService.get<string>('SOAP_DGP_USERNAME'),
          contrasenia: this.configService.get<string>('SOAP_DGP_PASSWORD'),
          hash: md5(
            this.configService.get<string>('SOAP_DGP_MD5_INI') +
              rfcNumEmp +
              this.configService.get<string>('SOAP_DGP_MD5_FIN') +
              this.configService.get<string>('SOAP_DGP_USERNAME'),
          ),
        },
        (err, res) => (err ? reject(err) : resolve(res)),
      );
    });
  }
}
