import * as fs from 'fs';
import {
  BadRequestException,
  Controller,
  Get,
  Post,
  Query,
  Response,
  UploadedFile,
  UseInterceptors,
} from '@nestjs/common';
import { FileInterceptor } from '@nestjs/platform-express';
import { ApiTags } from '@nestjs/swagger';
import { UploadFileService } from './upload-file.service';
import { IdInstitucionDto } from '../dto/id-institucion.dto';

@Controller('upload-file')
@ApiTags('upload-file')
export class UploadFileController {
  constructor(private uploadFileService: UploadFileService) {}

  @Post('carga-masiva-equipos')
  @UseInterceptors(FileInterceptor('csv'))
  cargaMasivaEquipos(
    @UploadedFile() file: Express.Multer.File,
    @Query() query: IdInstitucionDto,
  ) {
    const path = file ? `${file.destination}/${file.filename}` : null;

    if (!file) throw new BadRequestException('No se mandó ningún archivo.');
    return this.uploadFileService
      .createEquipos(path, parseInt(query.id_institucion))
      .then((res) => {
        fs.unlink(path, (err) => {});
        return res;
      });
  }

  @Post('carga-masiva-usuarios')
  @UseInterceptors(FileInterceptor('csv'))
  cargaMasivaUsuarios(
    @UploadedFile() file: Express.Multer.File,
    @Query() query: IdInstitucionDto,
  ) {
    const path = file ? `${file.destination}/${file.filename}` : null;

    if (!file) throw new BadRequestException('No se mandó ningún archivo.');
    return this.uploadFileService
      .createUsuarios(path, parseInt(query.id_institucion))
      .then((res) => {
        fs.unlink(path, (err) => {});
        return res;
      });
  }

  @Get('download-logo')
  downloadLogo(@Response() res, @Query() query: IdInstitucionDto) {
    return this.uploadFileService
      .downloadLogo(parseInt(query.id_institucion))
      .then((logo) => res.download(logo));
  }

  @Get('download-plantilla-equipos')
  downloadPlantillaEquipos(@Response() res) {
    return res.download('./upload/plantilla_equipos.csv');
  }

  @Get('download-plantilla-usuarios')
  downloadPlantillaUsuarios(@Response() res) {
    return res.download('./upload/plantilla_alumnos.csv');
  }

  @Post('upload-logo')
  @UseInterceptors(FileInterceptor('logo'))
  uploadLogo(
    @UploadedFile() file: Express.Multer.File,
    @Query() query: IdInstitucionDto,
  ) {
    return this.uploadFileService.uploadLogo(
      file,
      parseInt(query.id_institucion),
    );
  }
}
