import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { InstitucionCarrera } from '../institucion-carrera/entity/institucion-carrera.entity';
import { TipoUsuario } from '../tipo-usuario/entity/tipo-usuario.entity';
import { Usuario } from './entity/usuario.entity';
import { BcryptService } from '../bcrypt/bcrypt.service';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionCarreraService } from '../institucion-carrera/institucion-carrera.service';
import { InstitucionUsuarioService } from '../institucion-usuario/institucion-usuario.service';
import { NodemailerService } from '../nodemailer/nodemailer.service';
import { SoapClientService } from '../soap-client/soap-client.service';
import { TipoUsuarioService } from '../tipo-usuario/tipo-usuario.service';
import { query } from 'express';

@Injectable()
export class UsuarioService {
  constructor(
    @InjectRepository(Usuario) private repository: Repository<Usuario>,
    private bcryptService: BcryptService,
    private institucionService: InstitucionService,
    private institucionCarreraService: InstitucionCarreraService,
    private institucionUsuarioService: InstitucionUsuarioService,
    private nodemailerService: NodemailerService,
    private soapClientService: SoapClientService,
    private tipoUsuarioService: TipoUsuarioService,
  ) {}

  async create(
    usuario: string,
    nombre: string,
    tipoUsuario: TipoUsuario,
    institucionCarrera: InstitucionCarrera,
  ) {
    return this.repository
      .save(
        this.repository.create({
          nombre,
          tipoUsuario,
          usuario,
        }),
      )
      .then((usuario) =>
        this.institucionUsuarioService.create(institucionCarrera, usuario),
      );
  }

  correoPassword(password: string) {
    return `<h2>Estimad@ usuari@ del servicio de préstamo PC Puma, su registro se realizó con éxito</h2>
<p>Para acceder al servicio debe hacerlo con los siguientes datos:</p>

<p>•	Usuario: Número de cuenta o número de trabajador</p>

<p>•	Contraseña: ${password}</p>

<p style=" text-align: justify; text-justify: inter-word;">
  El presente registro le permite hacer uso del servicio de préstamo de dispositivos PC Puma
  seleccionados, el cual puede ser utilizado únicamente al interior de la entidad donde se solicita el
  servicio. Para acceder al servicio de préstamo es obligatorio y es su responsabilidad leer los
  “lineamientos PC- Puma” de la entidad en la que es solicitado el préstamo, ubicados en el módulo
  de préstamo y en la página de su entidad. El desconocimiento de dichos lineamientos no le exime
  de su responsabilidad y aplicación efectiva de los mismos. 
</p>

<p>Para mayor información o duda contáctenos por este medio.</p>

<p>Atte.</p>

<p>Pc Puma Solicita</p>

<p style=" text-align: justify; text-justify: inter-word;">
  <i>
    Hacemos de su conocimiento que la
    presente información es confidencial y será tratada como tal, es de uso exclusivo para el usuario al
    que fue asignada y se encuentra protegida por la normatividad de la Universidad Nacional
    Autónoma de México en la materia, cualquier uso no autorizado o indebido se hará del
    conocimiento de la autoridad universitaria. Puede consultar nuestro aviso de privacidad en:
    <a href="https://sdi.unam.mx/avisospriv.php" target="_blank">https://sdi.unam.mx/avisospriv.php</a>
  </i>
</p>
`;
  }

  async DGAE(usuario: string, id_institucion_carrera: number) {
    const institucionCarrera = await this.institucionCarreraService.findById(
      id_institucion_carrera,
    );

    return this.findByUsuario(usuario, true).then(async (existeUsuario) => {
      /* Falta conexión a soap */
      if (existeUsuario.password)
        throw new ConflictException('Ya fue regisrado este número de cuenta.');
      // if (!existeUsuario.activo)
      //   throw new ConflictException(
      //     'No se puede registar con este número de cuenta por que se encuentra desactivado.',
      //   );
      return this.institucionUsuarioService.findByUsuarioInstitucionCarrera(
        existeUsuario,
        institucionCarrera,
        'cuenta',
      );
    });
  }

  async DGP(id_institucion: number, usuario: string, rfc: string) {
    const institucionCarrera =
      await this.institucionCarreraService.findByIdInstitucionIdCarrera(
        id_institucion,
        1,
      );

    return this.findByUsuario(usuario, true).then(async (existeUsuario) => {
      /* Falta conexión a soap */
      if (existeUsuario.password)
        throw new ConflictException(
          'Ya fue regisrado este número de trabajador.',
        );
      // if (!existeUsuario.activo)
      //   throw new ConflictException(
      //     'No se puede registar con este número de trabajador por que se encuentra desactivado.',
      //   );
      return this.institucionUsuarioService.findByUsuarioInstitucionCarrera(
        existeUsuario,
        institucionCarrera,
        'trabajador',
      );
    });
  }

  async findAll(filtros: {
    pagina: string;
    id_carrera?: string;
    id_institucion?: string;
    id_tipo_usuario?: string;
    nombre?: string;
    usuario?: string;
  }) {
    const carrera = filtros.id_carrera
      ? await this.institucionCarreraService.findCarreraByIdCarrera(
          parseInt(filtros.id_carrera),
        )
      : null;
    const institucion = filtros.id_institucion
      ? await this.institucionService.findById(parseInt(filtros.id_institucion))
      : null;
    const tipoUsuario = filtros.id_tipo_usuario
      ? await this.tipoUsuarioService.findById(
          parseInt(filtros.id_tipo_usuario),
        )
      : null;
    const query = this.repository
      .createQueryBuilder('u')
      .innerJoinAndSelect('u.instituciones', 'is')
      .innerJoinAndSelect('u.tipoUsuario', 'tu')
      .innerJoinAndSelect('is.institucionCarrera', 'ic')
      .innerJoinAndSelect('ic.carrera', 'c')
      .innerJoinAndSelect('ic.institucion', 'i')
      .innerJoinAndSelect('c.nivel', 'n')
      .take(25)
      .skip((parseInt(filtros.pagina) - 1) * 25)
      .where('u.password IS NOT NULL');

    if (filtros.usuario)
      query.andWhere('u.usuario LIKE :usuario', {
        usuario: `%${filtros.usuario}%`,
      });
    if (filtros.nombre)
      query.andWhere('u.nombre LIKE :nombre', {
        nombre: `%${filtros.nombre}%`,
      });
    if (carrera)
      query.andWhere('c.id_carrera = :id_carrera', {
        id_carrera: carrera.id_carrera,
      });
    if (institucion)
      query.andWhere('i.id_institucion = :id_institucion', {
        id_institucion: institucion.id_institucion,
      });
    if (tipoUsuario)
      query.andWhere('tu.id_tipo_usuario = :id_tipo_usuario', {
        id_tipo_usuario: tipoUsuario.id_tipo_usuario,
      });
    return query.getManyAndCount();
  }

  findById(id_usuario: number, validarNoExiste = true, password = false) {
    return this.repository
      .findOne({
        join: {
          alias: 'u',
          innerJoinAndSelect: {
            is: 'u.instituciones',
            ic: 'is.institucionCarrera',
            c: 'ic.carrera',
            i: 'ic.institucion',
            n: 'c.nivel',
          },
        },
        where: { id_usuario },
      })
      .then((usuario) => {
        if (validarNoExiste && (!usuario || (password && !usuario.password)))
          throw new NotFoundException('No existe este usuario.');
        return usuario;
      });
  }

  findByUsuario(
    usuario: string,
    validarNoExiste = true,
    password = false,
    institucionesActivas = false,
  ) {
    const query = this.repository
      .createQueryBuilder('u')
      .innerJoinAndSelect('u.instituciones', 'is')
      .innerJoinAndSelect('u.tipoUsuario', 'tu')
      .innerJoinAndSelect('is.institucionCarrera', 'ic')
      .innerJoinAndSelect('ic.carrera', 'c')
      .innerJoinAndSelect('ic.institucion', 'i')
      .innerJoinAndSelect('c.nivel', 'n')
      .where('usuario = :usuario', { usuario });

    if (institucionesActivas) query.andWhere('is.activo = 1');
    return query.getOne().then((usuario) => {
      if (validarNoExiste && (!usuario || (password && !usuario.password)))
        throw new NotFoundException('No existe este usuario.');
      return usuario;
    });
  }

  passwordReset(id_usuario: number) {
    const password = this.bcryptService.generarPassword();

    return this.findById(id_usuario, true, true)
      .then((usuario) => {
        usuario.password = this.bcryptService.encriptar(password);
        return this.repository.save(usuario);
      })
      .then((usuario) =>
        this.nodemailerService.sendEmail({
          email: `${usuario.usuario}@pcpuma.acatlan.unam.mx`,
          subject: 'Credenciales Pc Puma',
          html: this.correoPassword(password),
        }),
      )
      .then((_) => ({
        message: 'Se cambió correctamente la constraseña del usuario.',
      }));
  }

  registrar(id_institucion_usuario: number, telefono: string, correo?: string) {
    const password = this.bcryptService.generarPassword();
    let message =
      'Se creó correctamente tu cuenta, ingresa a tu correo institucional y consulta tu contraseña para acceder a este servicio.';

    return this.institucionUsuarioService
      .findById(id_institucion_usuario)
      .then((institucionUsuario) => {
        const institucion = institucionUsuario.institucionCarrera.institucion;
        const usuario = institucionUsuario.usuario;

        if (usuario.password)
          throw new ConflictException(
            'Ya fue registrado este número de cuenta.',
          );
        if (!institucion.email_institucional) {
          if (!correo)
            throw new ConflictException(
              'No se envió un correo electrónico para el regístro.',
            );
          else usuario.correo = correo;
          message =
            'Se creó correctamente tu cuenta, ingresa a tu correo y consulta tu contraseña para acceder a este servicio.';
        } else usuario.correo = `${usuario.usuario}${institucion.dominio}`;
        usuario.telefono = telefono;
        usuario.password = this.bcryptService.encriptar(password);
        return this.repository.save(usuario);
      })
      .then((usuario) =>
        this.nodemailerService.sendEmail({
          email: usuario.correo,
          subject: 'Credenciales Pc Puma',
          html: this.correoPassword(password),
        }),
      )
      .then((_) => ({ message }));
  }

  update(attrs: Partial<Usuario>, validarPassword = true) {
    return this.findById(attrs.id_usuario, true, validarPassword)
      .then((usuario) => {
        Object.assign(usuario, attrs);
        return this.repository.save(usuario);
      })
      .then((_) => ({
        message: 'Se guardaron los cambios correctamente.',
      }));
  }
}
